$NOMOD51
$INCLUDE (reg51.inc)

; ***** BEGIN LICENSE BLOCK *****
; Version: MPL 1.1
;
; The contents of this file are subject to the Mozilla Public License Version 
; 1.1 (the "License"); you may not use this file except in compliance with 
; the License. You may obtain a copy of the License at 
; http://www.mozilla.org/MPL/
;
; Software distributed under the License is distributed on an "AS IS" basis,
; WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
; for the specific language governing rights and limitations under the
; License.
;
; The Original Code is firmware for the F80DMX device.
;
; The Initial Developer of the Original Code is
; Tomsoft.
; Portions created by Tomsoft are Copyright (C) 2009
; Tomsoft. All Rights Reserved.
;
; Contributor(s):
;   Tommy Sools <tom@sools.com>
;
; ***** END LICENSE BLOCK *****

; TODO implement a DMX timer function.

; V1.1
; improved addressing macro

; V1.0
; Initial version.

; 16 MHz quartz = 16000000 Hz
; 1 pulse = 0,0000000625 seconds
; 1 cycle = 0,0000000625 seconds * 12 pulses = 0,00000075 seconds = 0,00075 milliseconds
; 16 bit timer is being used, meaning every 65536 cycles it will overflow.
; 65536 cycles take 65536 * 0,00075 milliseconds = 49,152 milliseconds.
; The DMX signal can only be considered to be lost after 1 second = 1000 milliseconds =>
; only after 1000 milliseconds / 49,152 milliseconds = 20,345052083333333333333333333333 timer overflows ~ 21 timer overflows.

; Fogger can maintain output for approx. 42 seconds before heating up again.

; Slot 1: duration setting (linear)

;   0 -  19 unenforced stop (possibly interrupted by smoking collected from "forced smoking"
;  20 -  38 forced stop
;  39       timed 1 sec. (not yet implemented: will result in forced stop)
;  95       timed 15 sec. (not yet implemented: will result in forced stop)
; 155       timed 30 sec. (not yet implemented: will result in forced stop)
; 215       timed 45 sec. (not yet implemented: will result in forced stop)
; 216 - 235 unenforced smoking (never prolonged)
; 236 - 255 forced smoking (possibly prolonged)

; Slot 2: interval minutes setting (linear, not yet implemented: no effect)

;   0 -  18 no interval
;  19       59 min.
;  75       45 min.
; 135       30 min.
; 195       15 min.
;  255      0 min.

; Slot 3: interval seconds setting (linear, not yet implemented: no effect)

;   0 -  18 no interval
;  19       59 sec.
;  75       45 sec.
; 135       30 sec.
; 195       15 sec.
;  255      0 sec.

		NAME		f80dmx

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_ledpower		EQU	P3.1	; power LED
_ledheatingready	EQU	P3.2	; heating element ready LED
_ledpumpactive		EQU	P3.4	; pump active LED
_heating		EQU	P3.3	; smoke machine heating detection (steady = ready; changing approx. every 10 milliseconds = still heating; causes external interrupt 1 about every 10 milliseconds)
_pumping		EQU	P3.5	; smoke machine pump activation (0 = active; 1 = inactive)
_inaddressl 		EQU	P1     	; low order address setting through dipswitches
_inaddressh		EQU	P3.7   	; high order address setting
			       		; (1 = 1; 2 = 2; 3 = 4; 4 = 8; 5 = 16; 6 = 32; 7 = 64; 8 = 128; 9 = 256; 10 = terminator)
					; all dipswitches off before powering up = service mode until power down
					; all dipswitches off only after powering up = address 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dmxpointer		EQU	R0   		; DMX value pointer OK
dmxcounter		EQU	R1   		; local DMX table counter
signallostcountdown	EQU	R2
smokecountdownh		EQU	R3
smokecountdownl		EQU	R4
tempaddressh		EQU	R5
tempaddressl		EQU	R6
loopcountdown		EQU	R7

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

tablesize		EQU	4	; size of DMX table (break + start code + 2 DMX values)
signallostcountdowninit	EQU	21	; number of times the timer must overflow before the signal will be considered to be lost

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?STACK		SEGMENT	IDATA		; ?STACK goes into IDATA RAM.
		RSEG	?STACK		; switch to ?STACK segment.
		DS	20

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?BIT		SEGMENT	BIT		; segment for BIT RAM.
	   	RSEG	?BIT		; switch to ?BIT segment
dmxsignalcheck: 	DBIT	1	; DMX signal received bit
dmxsignal: 		DBIT	1	; DMX signal present bit
heatingsignalcheck:	DBIT	1	; heating signal received bit
heatingsignal:		DBIT	1	; heating signal present bit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?DATA		SEGMENT	DATA		; segment for DATA RAM.
	   	RSEG	?DATA		; switch to this data segment
initaddressh:		DS	1		; initial processed high address byte
initaddressl:		DS	1		; initial processed low address byte
delay1:			DS	1
delay2:			DS	1
delay3:			DS	1
table:   		DS	tablesize	; reserve bytes for DMX table
addressh:		DS	1		; address high byte
addressl:		DS	1		; address low byte

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%*DEFINE (ADDRESS_REFRESH) LOCAL addressh_enabled, address_not_zero, address_end (
		MOV	tempaddressh,#1
		JNB	_inaddressh,%addressh_enabled	; retrieve high address byte
		MOV	tempaddressh,#0
%addressh_enabled:
		MOV	A,_inaddressl        		; retrieve low address byte
		CPL	A
		MOV	tempaddressl,A
		JNZ	%address_not_zero		; check if low address byte is zero
		CJNE	tempaddressh,#0,%address_not_zero	; check if high address byte is zero
		; the full address is zero, so use address 1
		SJMP	%address_end

%address_not_zero:	; the full address is not zero
; Decrement low address byte by 1, so that the address set using the dipswitches will be one-based rather than zero-based.
; This way a zero setting for the address could be used for special purposes.
; For address 1 (dipswitch 1 enabled): initaddressh = 0; initaddressl = 0
; For address 2 (dipswitch 2 enabled): initaddressh = 0; initaddressl = 1
; For address 3 (dipswitch 1 & 2 enabled): initaddressh = 0; initaddressl = 2
; etc.
		CLR	C
		SUBB	A,#1
		MOV	tempaddressl,A
		MOV	A,tempaddressh
		SUBB	A,#0
		MOV	tempaddressh,A

%address_end:	MOV	initaddressh,tempaddressh
		MOV	initaddressl,tempaddressl
)

%*DEFINE (VISUALIZE_HEATING) LOCAL setledheatingready, checkheatingreadyend (
		JNB	heatingsignal,%setledheatingready
		SETB	_ledheatingready	; reset heating ready LED
		SJMP	%checkheatingreadyend
%setledheatingready:
		CLR	_ledheatingready	; set heating ready LED
%checkheatingreadyend:
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0			; absolute Segment at Address 0
		LJMP	start			; reset location (jump to start)

?CODE		SEGMENT	CODE
		RSEG	?CODE			; switch to this code segment

start:          MOV	SP,#?STACK-1   		; assign stack at beginning

init:		CLR	dmxsignal
		CLR	dmxsignalcheck
		SETB	heatingsignal
		CLR	heatingsignalcheck

		MOV	signallostcountdown,#signallostcountdowninit

		; check if the full address is zero
		JNB	_inaddressh,address_not_zero
		MOV	A,_inaddressl
		CJNE	A,#255,address_not_zero

		; address is zero, so go into service mode
		LJMP	service_mode

address_not_zero:
		%ADDRESS_REFRESH

		MOV	dmxpointer,#table+tablesize	; initialize DMX value pointer
emptytable:	DEC	dmxpointer
		MOV	@dmxpointer,#0
		CJNE	dmxpointer,#table,emptytable

		MOV     SCON,#10001000B		; mode 2 (9-bit UART, fixed baud rate),
						; no automatic address recognition, serial reception disabled,
						; set send bit 8, unset receive bit 8,
						; unset transmit and receive interrupt flags

		SETB	PS			; set high priority for serial port interrupt
		SETB    ES	       		; enable serial interrupt
		SETB	IT1			; external interrupt 1 will be falling edge triggered

		SETB	PX1			; set high priority for external interrupt 1
		SETB	EX1			; enable external interrupt 1

;		ORL	TMOD,#00010001B		; set timer 1 to 16-bit timer; set timer 0 to 16-bit timer
		ORL	TMOD,#0000001B		; set timer 0 to 16-bit timer

;		MOV	TH1,0			; initialize timer 1 count
;		MOV	TL1,0

;		SETB	ET1	       		; enable timer 1 interrupt
		SETB	ET0	       		; enable timer 0 interrupt

		SETB    EA	      		; enable global interrupts
		SETB	REN			; enable serial reception
;		SETB	TR1			; enable timer 1
		SETB	TR0			; enable timer 0

		MOV	smokecountdownh,#0
		MOV	smokecountdownl,#0
		CLR	_ledpower		; set power LED

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

loop:
		%ADDRESS_REFRESH
		MOV	P0,initaddressh
		MOV	P2,initaddressl
		%VISUALIZE_HEATING
		SJMP	loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		; approx. 500 millseconds delay ==> 666666,66666666666666666666666667 cycles are neccesary
		; total 716297 cycles ==> 537,22275 milliseconds
%*DEFINE (DELAY) LOCAL ldelay, ldelay2, ldelay3 (
		MOV	delay1,#255	; 2 cycles

%ldelay:	MOV	delay2,#255	; 2 cycles * 255 = 510

%ldelay2:	NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		DJNZ	delay2,%ldelay2	; 2 cycles * 255 * 255 = 130050
		DJNZ	delay1,%ldelay	; 2 cycles * 255 = 510
)
%*DEFINE (TESTDIP (dipbit)) (
		JB	%dipbit,$		; wait for dipswitch to be switched on
		CLR	_ledheatingready	; set heating ready led
		%DELAY
		SETB	_ledheatingready	; reset heating ready led

		JNB	%dipbit,$		; wait for dipswitch to be switched off
		CLR	_ledheatingready	; set heating ready led
		%DELAY
		SETB	_ledheatingready	; reset heating ready led
)

service_mode:	; slight delay before testing LEDs
		%DELAY

		; test LEDs
		MOV	loopcountdown,#9
service_led_loop:
		%DELAY
		SETB	_ledpower		; reset power LED
		CLR	_ledpumpactive		; set pump active LED
		%DELAY
		SETB	_ledpumpactive		; reset pump active LED
		CLR	_ledheatingready	; set heating ready LED
		%DELAY
		SETB	_ledheatingready	; reset heating ready LED
		CLR	_ledpower		; set power LED
		DJNZ	loopcountdown,service_led_loop

		; test dipswitches. the user will have to switch the dipswitches on and off one after another,
		; starting with dipswitch 1 and ending with dipswitch 9.
		%TESTDIP (_inaddressl.0)
		%TESTDIP (_inaddressl.1)
		%TESTDIP (_inaddressl.2)
		%TESTDIP (_inaddressl.3)
		%TESTDIP (_inaddressl.4)
		%TESTDIP (_inaddressl.5)
		%TESTDIP (_inaddressl.6)
		%TESTDIP (_inaddressl.7)
		%TESTDIP (_inaddressh)

service_init:	SETB	IT1			; external interrupt 1 will be falling edge triggered

		SETB	PX1			; set high priority for external interrupt 1
		SETB	EX1			; enable external interrupt 1

		ORL	TMOD,#00000001B		; set timer 0 to 16-bit timer (001H)

		SETB	ET0	       		; enable timer 0 interrupt

		SETB    EA	      		; enable global interrupts
		SETB	TR0			; enable timer 0

service_loop:	%VISUALIZE_HEATING

service_checkpumping:
		JNB	_inaddressl.0,service_setpumping	; depending on dipswitch 1, activate or inactivate pump
service_resetpumping:
		SETB	_pumping		; deactivate pump
		SETB	_ledpumpactive		; reset pump active LED
		SJMP	service_checkpumpingend
service_setpumping:
		CLR	_ledpumpactive		; set pump active LED
		CLR	_pumping		; activate pump
service_checkpumpingend:

		SJMP	service_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0BH	      		; 0BH is address for Timer 0 interrupt			;!3 cycles this far
		LJMP	timer0int							     		;!2 cycles

?TIMER0		SEGMENT	CODE
		RSEG	?TIMER0

timer0int:	PUSH	PSW	     		; store PSW for safekeeping				;!2 cycles
		PUSH	ACC	      		; store ACC for safekeeping				;!2 cycles

		MOV	C,dmxsignalcheck
		MOV	dmxsignal,C
		CLR	dmxsignalcheck
		JC	blinkpowerled
enablepowerled:	CLR	_ledpower		; set power LED to indicate that there is no DMX signal going on
		MOV	A,signallostcountdown
		JZ	checkpowerledend
		DEC	signallostcountdown
		SJMP	checkpowerledend
blinkpowerled:	MOV	signallostcountdown,#signallostcountdowninit
		CPL	_ledpower		; reverse power LED's status for DMX signal indication
checkpowerledend:

		MOV	C,heatingsignalcheck
		MOV	heatingsignal,C
		CLR	heatingsignalcheck

		; check if any DMX signal is present at all
		MOV	A,signallostcountdown
		JZ	signallost

		; check whether still heating
		JB	heatingsignal,stillheating

                ; not still heating, so check whether smoke should be output depending on the DMX value
		MOV	dmxcounter,#table+2
		CJNE	@dmxcounter,#215,check216a
check216a:	JNC	smokeoutput		; if @dmxcounter is greater than or equal to #216, do forced or unenforced smoking
		CJNE	@dmxcounter,#19,check20a
check20a:	JNC	nosmokeoutput		; if @dmxcounter is greater than or equal to #20 (up to 38 eventually), do forced stop

		; @dmxcounter is greater than or equal to #0, so do unenforced stop
decrementsmokecountdowncheck:
		CJNE	smokecountdownh,#0,decrementsmokecountdown
		CJNE	smokecountdownl,#0,decrementsmokecountdown
		SJMP	nosmokeoutput

decrementsmokecountdown:
		; smokecountdown is not zero, so decrement smokecountdown and activate pump
		CLR	C
		MOV	A,smokecountdownl
		SUBB	A,#1
		MOV	smokecountdownl,A
		MOV	A,smokecountdownh
		SUBB	A,#0
		MOV	smokecountdownh,A
smokeoutput:	; activate pump
		CLR	_ledpumpactive		; set pump active LED
		CLR	_pumping		; activate pump
		SJMP	timer0int_end

stillheating:	
		MOV	dmxcounter,#table+2
		CJNE	@dmxcounter,#235,check236b
check236b:	JNC	incrementsmokecountdowncheck	; if @dmxcounter is greater than or equal to #236, do forced smoking

		; @dmxcounter is greater than or equal to #0, so do unenforced smoking or forced or unenforced stop
		SJMP	nosmokeoutput

		; smoke is requested but is not possible at the moment, so increment smokecountdown,
		; but only if it isn't about to overflow yet, otherwise do nothing
incrementsmokecountdowncheck:
		CJNE	smokecountdownh,#255,incrementsmokecountdowncheckl
		SJMP	nosmokeoutput
incrementsmokecountdowncheckl:
		CJNE	smokecountdownl,#255,incrementsmokecountdown
		SJMP	nosmokeoutput

incrementsmokecountdown:
		MOV	A,smokecountdownl
		ADD	A,#1
		MOV	smokecountdownl,A
		MOV	A,smokecountdownh
		ADDC	A,#0		
		MOV	smokecountdownh,A
		SJMP	nosmokeoutput

signallost:	; no DMX signal is present, so immediately deactivate pump and reset smokecountdown
		MOV	smokecountdownh,#0
		MOV	smokecountdownl,#0

nosmokeoutput:	; deactivate pump
		SETB	_ledpumpactive		; reset pump active LED
		SETB	_pumping		; deactivate pump

timer0int_end:	POP	ACC	      		; restore ACC						;!2 cycles
		POP	PSW	      		; restore PSW						;!2 cycles
		RETI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;	CSEG	AT  1BH	      		; 1BH is address for Timer 1 interrupt			;!3 cycles this far
;	LJMP	timer1int							     		;!2 cycles
;
;?TIMER1		SEGMENT	CODE
;		RSEG	?TIMER1
;
;timer1int:	PUSH	PSW	     		; store PSW for safekeeping				;!2 cycles
;		PUSH	ACC	      		; store ACC for safekeeping				;!2 cycles
;
;
;timer1int_end:	POP	ACC	      		; restore ACC						;!2 cycles
;		POP	PSW	      		; restore PSW						;!2 cycles
;		RETI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  13H	     			; 13H is the address for External interrupt 1		; 3 cycles this far
		LJMP	extint1											; 2 cycles

?SERIAL		SEGMENT	CODE
		RSEG	?SERIAL

extint1:	PUSH	PSW	      			; store PSW for safekeeping				; 2 cycles
		PUSH	ACC	     			; store ACC for safekeeping				; 2 cycle

		SETB	heatingsignalcheck

		POP	ACC				; restore ACC				      		; 2 cycles
		POP	PSW				; restore PSW				      		; 2 cycles
		RETI								       				; 2 cycles

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  23H	     			; 23H is the address for Serial interrupt		; 3 cycles this far
;		LJMP	serialint											; 2 cycles

;?SERIAL		SEGMENT	CODE
;		RSEG	?SERIAL

serialint:	PUSH	PSW	      			; store PSW for safekeeping				; 2 cycles
		PUSH	ACC	     			; store ACC for safekeeping				; 2 cycle

                JNB	RI,serialint_end     									; 2 cycles

		SETB	dmxsignalcheck										; 1 cycle

                JNB	RB8,dmxbreak										; 2 cycles

		; if counter is zero then skip
		CJNE	dmxpointer,#table,not_zero								; 2 cycles
		SJMP	serialint_end										; 2 cycles
		
dmxbreak: 	; break detected
		MOV	dmxpointer,#table+1						     			; 1 cycle

		MOV	addressh,initaddressh		; retrieve high order address byte			; 2 cycles
		MOV	addressl,initaddressl		; retrieve low order address byte			; 2 cycles

		SJMP	serialint_end								       		; 2 cycles

not_zero:      	; if counter exceeds the table's size then skip
		CJNE	dmxpointer,#table+tablesize,dmxlimitcheck	      					; 2 cycles

dmxlimitcheck:	JNC	serialint_end										; 2 cycles

		CJNE	dmxpointer,#table+1,nostartbyte	; check if this is a start byte being received		; 2 cycles

		; this is a start byte being received
		MOV	A,SBUF				; retrieve start byte					; 1 cycle
		JZ	goodstartbyte                   ; the received start byte is zero			; 2 cycles

badstartbyte:	MOV	dmxpointer,#table		; the received start byte is not zero, reset pointer	; 1 cycle
		SJMP	serialint_end										; 2 cycles

goodstartbyte:	INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end

nostartbyte:	MOV	A,addressh										; 1 cycle
		JNZ	decrement										; 2 cycles
		MOV	A,addressl										; 1 cycle
		JNZ	decrement										; 2 cycles

		MOV	@dmxpointer,SBUF		; store DMX value					; 2 cycles

		INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end			; counter is already zero, so don't decrement again	; 2 cycles

decrement:	CLR	C											; 1 cycle
		MOV	A,addressl										; 1 cycle
		SUBB	A,#1											; 1 cycle
		MOV	addressl,A										; 1 cycle
		MOV	A,addressh										; 1 cycle
		SUBB	A,#0	     										; 1 cycle
		MOV	addressh,A								       		; 1 cycle

serialint_end:	CLR	RI				; serial receive data handled		      		; 1 cycle
		POP	ACC				; restore ACC				      		; 2 cycles
		POP	PSW				; restore PSW				      		; 2 cycles
		RETI								       				; 2 cycles

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		END			; End Of File

